@@ -26,6 +26,7 @@ __all__ = [
     "ParameterList",
     "ParameterDict",
     "auto_repr",
+    "set_state_dict_pass_attr",
 ]
 
 _ForwardHookType = Callable[["Module", tuple[Tensor], tuple[Tensor]], None]
@@ -51,6 +52,8 @@ class Module:
         self._forward_hooks: list[_ForwardHookType] = []
         self._backward_hooks: list[_BackwardHookType] = []
 
+        self._state_dict_pass_attr = set()
+
     def __setattr__(self, name: str, value: Any) -> None:
         registry_map: dict[Type, OrderedDict[str, Any]] = {
             nn.Parameter: self._parameters,
@@ -76,6 +79,9 @@ class Module:
 
         super().__setattr__(name, value)
 
+    def setattr_raw(self, name: str, value: Any) -> None:
+        object.__setattr__(self, name, value)
+
     def add_module(self, name: str, module: Self) -> None:
         if not isinstance(module, Module) and module is not None:
             raise TypeError(f"{module} is not a Module.")
@@ -198,6 +204,10 @@ class Module:
                 destination=destination, prefix=prefix + name + ".", keep_vars=keep_vars
             )
 
+        for key in destination.keys():
+            if key in self._state_dict_pass_attr:
+                del destination[key]
+
         return destination
 
     def load_state_dict(self, state_dict: OrderedDict, strict: bool = True) -> None:
@@ -303,6 +313,14 @@ def auto_repr(*attr_names: str) -> Callable[[T], T]:
     return wrapper
 
 
+def set_state_dict_pass_attr(*attr_names: str) -> Callable[[T], T]:
+    def wrapper(cls: T) -> T:
+        cls._state_dict_pass_attr = set(attr_names)
+        return cls
+
+    return wrapper
+
+
 class Sequential(Module):
     @overload
     def __init__(self, *modules: Module) -> None: ...
